from netgen.geom2d import unit_square
from ngsolve import *

mesh = Mesh(unit_square.GenerateMesh(maxh=0.2))

# interpolation points [ (x,y,value) ]
data = [ (0.1,0.1, 1), (0.5,0.5,-1) ]

fes = H1(mesh, order=1)
u,v = fes.TnT()

# lease squares with H1-regularization
reg = 1e-3
a = BilinearForm(reg*grad(u)*grad(v)*dx+reg*u*v*dx).Assemble()

f = LinearForm(fes)
for x,y,val in data:
    # search for point in mesh
    # mesh-point mp has element number, and coords in the reference el    
    mp = mesh(x,y)
    ei = ElementId(mp.nr)
    fel = fes.GetFE(ei)
    shapes = fel.CalcShape(*mp.pnt)
    dofs = fes.GetDofNrs(ei)
    print ("dofs:", list(dofs), "shapes:", list(shapes))
    for s,d in zip(shapes, dofs):
        f.vec[d] += val*s
    for s1,d1 in zip(shapes, dofs):
        for s2,d2 in zip(shapes, dofs):
            a.mat[d1,d2] += s1*s2
        
gfu = GridFunction(fes)
gfu.vec.data = a.mat.Inverse() * f.vec
Draw (gfu)
